/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.MessageSender
 *
 * de.bsvrz.kex.tls.osi7.MessageSender is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.MessageSender is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.MessageSender.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.MessageSender.
 *
 *  de.bsvrz.kex.tls.osi7.MessageSender ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.MessageSender wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.osi7;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

//~ KLASSEN ===================================================================

/**
 * Klasse zur Verwaltung und zum Versenden von Betriebsmeldungen an eine Betriebsmeldungsverwaltung.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 */
public class MessageSender {

    /**
     * Das Singleton wird in jedem Fall erzeugt.
     */
    private static MessageSender INSTANCE = new MessageSender();

    /**
     * DebugLogger fr Debug-Ausgaben
     */
    private static Debug _debug;

    //~ FELDER ================================================================

    /* ############### Attribute der Attributgruppe atg.betriebsMeldung ################### */

    /**
     * ID der Applikation, die die Informationsmeldung erzeugt hat.
     */
    private long _applicationID;

    /**
     * Fr jede Applikation eindeutige Kennung, die nach einem Neustart gleich bleibt.
     */
    private String _applicationLabel;

    /**
     * Typ der Applikation, die die Informationsmeldung erzeugt hat.
     */
    private SystemObjectType _applicationType;

    /**
     * Die Urlasserinformation, falls keine bergeben wird. Sie besteht aus dem lokalen Benutzer, der sich beim
     * Datenverteiler angemeldet hat und ohne die Angabe einer Ursache und eines Veranlassers.
     */
    private MessageCauser _causer;

    /**
     * LaufendeNummer: Laufende Nummerierung der durch die Applikation seit dem Applikationsstart erzeugten
     * Informationsmeldungen. Aus der Kombination von ApplikationsID und LaufendeNummer lsst sich prfen, ob Meldungen
     * verloren wurden.
     */
    private long _consecutiveNumber;

    /**
     * Die DataDescription zum Senden von Betriebsmeldungen.
     */
    private DataDescription _dataDescriptionSender;

    /**
     * Dieses Systemobjekt muss vom typ.betriebsMeldungsVerwaltung sein und wird zum Versenden der Betriebsmeldungen
     * eingesetzt.
     */
    private SystemObject _messageObject = null;

    /**
     * Das Sende-Objekt fr den Versand von Betriebsmeldungen.
     */
    private OperatingMessageSender _operatingMessageSender;

    /**
     * Speichert den Klassennamen der Applikation
     */
    private String _rootName;

    /**
     * Die Verbindung zum Datenverteiler.
     */
    private ClientDavInterface _connection = null;

	/** Flag, ob Objekt der Betriebsmeldungsverwaltung gltig ist */
	private boolean _isMessageObject = false;

	//~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /* ################# Methoden ################# */

    /**
     * Privater Konstruktor erzeugt ein leeres Objekt dieser Klasse.
     */
    private MessageSender() {
        _debug = Debug.getLogger();
        _debug.fine("Singleton wurde erstellt", this.getClass().getName());
    }

    //~ GET METHODEN ==========================================================

    /**
     * Gibt die Instanz dieser Klasse zurck. Ein neues Objekt wird erstellt, falls noch keines vorhanden ist.
     *
     * @return Objekt dieser Klasse
     */
    public static MessageSender getInstance() {
        return INSTANCE;
    }

    /**
     * Bestimmt zur Betriebsmeldung die Position im Code, an der die Betriebsmeldung rausgeschickt wurde. Angegeben wird
     * der Klassenname, der Methodenname und die Zeilennummer.
     *
     * @param t Throwable
     * @return die Aufrufposition der Betriebsmeldung
     */
    private String getCallPosition(Throwable t) {
        StringBuilder callPosition = new StringBuilder();

        if (t.getStackTrace().length > 1) {
            StackTraceElement traceElement = t.getStackTrace()[1];

            callPosition.append(_rootName);
            callPosition.append(" - ").append(traceElement.getClassName());
            callPosition.append(".").append(traceElement.getMethodName());
            callPosition.append("(").append(traceElement.getFileName());
            callPosition.append(": ").append(traceElement.getLineNumber());
            callPosition.append(")");
        } else {
            callPosition.append("Aufrufposition nicht ermittelbar!");
        }

        return callPosition.toString();
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiert das Objekt indem die Verbindung zum Datenverteiler und die Kennung der Applikation bergeben wird.
     *
     * @param connection       Verbindung zum Datenverteiler
     * @param applicationName  Name der Applikation
     * @param pidMessageSender eindeutige Kennung der Applikation, entweder PID eines Objekts vom Typ typ.betriebsMeldungsVerwaltung
     *                         oder die AOE (ist i.d.R. vom Typ typ.betriebsMeldungsVerwaltung), falls hier "" bergeben wird.
     */
    public void init(final ClientDavInterface connection, String applicationName, String pidMessageSender) {
        try {
            _messageObject = connection.getDataModel().getObject(pidMessageSender);

            if ((_messageObject == null) ||!_messageObject.isOfType("typ.betriebsMeldungsVerwaltung")) {

                // AOE versuchen...
                pidMessageSender = connection.getDataModel().getConfigurationAuthorityPid();
                _messageObject   = connection.getDataModel().getObject(pidMessageSender);

                if ((_messageObject == null) ||!_messageObject.isOfType("typ.betriebsMeldungsVerwaltung")) {
                    _debug.warning(String.format("Das Objekt mit der Pid %s besitzt nicht den Typ: BetriebsMeldungsVerwaltung. Betriebsmeldungen knnen nicht verschickt werden!", pidMessageSender));

                    return;
                }
            }

            _debug.config("Zum Senden wird folgendes BetriebsMeldungsVerwaltungsObjekt verwendet", _messageObject);
            _connection = connection;

            final ClientApplication application = _connection.getLocalApplicationObject();

            // Werte der Applikation speichern
            _applicationID     = application.getId();
            _consecutiveNumber = 0;
            _applicationType   = application.getType();
            _applicationLabel  = "KExTlsOsi7";
            _rootName          = applicationName;

            // Standard-Urlasser, falls keiner bergeben wurde
            _causer = new MessageCauser(_connection.getLocalUser(), "", "");

            // Datenidentifikation ermitteln und Anmeldung als Sender
            final DataModel      configuration = _connection.getDataModel();
            final AttributeGroup atg           = configuration.getAttributeGroup("atg.betriebsMeldung");
            final Aspect         asp           = configuration.getAspect("asp.information");

            _dataDescriptionSender  = new DataDescription(atg, asp);
            _operatingMessageSender = new OperatingMessageSender();
            _connection.subscribeSender(_operatingMessageSender, _messageObject, _dataDescriptionSender, SenderRole.sender());
	        _isMessageObject = true;
	        _debug.config("Der MessageSender wurde initialisiert und ist bereit zum Senden von Betriebsmeldungen.");
        }
        catch (OneSubscriptionPerSendData oneSubscriptionPerSendData) {
            _debug.error("Fehler", oneSubscriptionPerSendData);
            oneSubscriptionPerSendData.printStackTrace();
        }
    }

    /**
     * Gibt die laufende Nummer zurck und erhht sie um eins.
     *
     * @return die laufende Nummer
     */
    private long nextConsecutiveNumber() {
        return _consecutiveNumber++;
    }

    /**
     * Die vollstndige Betriebsmeldung. Sie enthlt alle Parameter, die fr eine Betriebsmeldung in Frage kommen knnen.
     *
     * @param id               ID der Meldung. Dieses Attribut kann von der Applikation gesetzt werden, um einen Bezug zu
     *                         einer vorherigen Meldung herzustellen.
     * @param type             der MeldungsTyp
     * @param messageTypeAddOn der MeldungsTypZusatz
     * @param grade            die MeldungsKlasse
     * @param referenceObject  Referenz auf ein beliebiges Konfigurationsobjekt, auf das sich die Meldung bezieht.
     * @param state            Gibt den Zustand einer Meldung an.
     * @param message          Text der Meldung
     */
    public void sendMessage(String id, MessageType type, String messageTypeAddOn, MessageGrade grade, SystemObject referenceObject, MessageState state, String message) {
        if (!_isMessageObject) {
            final String errorMessage = "Betriebsmeldungen knnen nicht verschickt werden. Die Initialisierung ist nicht erfolgt oder war fehlerhaft. Es gibt kein passendes BetriebsMeldungsVerwaltungs-Objekt. Evtl. besitzt der lokale Konfigurationsverantwortliche den Typ Betriebsmeldungsverwaltung nicht.";

            _debug.warning(errorMessage);

            return;
        }

        if ((messageTypeAddOn == null) || messageTypeAddOn.equals("")) {
            messageTypeAddOn = getCallPosition(new Throwable());
        }

        // versenden der Betriebsmeldung
        final Data data = _connection.createData(_dataDescriptionSender.getAttributeGroup());

        // erst die Werte, die durch die Initialisierung vorgegeben sind
        data.getUnscaledValue("ApplikationsID").set(_applicationID);
        data.getUnscaledValue("LaufendeNummer").set(nextConsecutiveNumber());
        data.getReferenceValue("ApplikationsTyp").setSystemObject(_applicationType);
        data.getTextValue("ApplikationsKennung").setText(_applicationLabel);

        // Werte, die durch sendMessage bergeben wurden
        data.getTextValue("ID").setText(id);
        data.getTextValue("MeldungsTyp").setText(type.getMessageType());
        data.getTextValue("MeldungsTypZusatz").setText(messageTypeAddOn);
        data.getTextValue("MeldungsKlasse").setText(grade.getGrade());

        if (referenceObject == null) {
            data.getReferenceArray("Referenz").setLength(0);
        } else {
            data.getReferenceArray("Referenz").setLength(1);
            data.getReferenceArray("Referenz").getReferenceValue(0).setSystemObject(referenceObject);
        }

        data.getTextValue("Status").setText(state.getState());

        // der Urlasser bentigt eine Extrabehandlung
        final Data causerData = data.getItem("Urlasser");

        causerData.getReferenceValue("BenutzerReferenz").setSystemObject(_causer.getUser());
        causerData.getTextValue("Ursache").setText(_causer.getCause());
        causerData.getTextValue("Veranlasser").setText(_causer.getCauser());
        data.getTextValue("MeldungsText").setText(message);

        if (_operatingMessageSender.getState() == 0) {
            try {
                final ResultData resultData = new ResultData(_messageObject, _dataDescriptionSender, System.currentTimeMillis(), data);

                _connection.sendData(resultData);
            }
            catch (SendSubscriptionNotConfirmed sendSubscriptionNotConfirmed) {
                _debug.error("Fehler", sendSubscriptionNotConfirmed);
                sendSubscriptionNotConfirmed.printStackTrace();
            }
            catch (Exception ex) {
                _debug.error("Fehler", ex);

                throw new RuntimeException(ex);
            }
        } else {
            _debug.warning("Senden ist nicht mglich, da die Sendesteuerung noch keine Sendeerlaubnis gibt! Evtl. fehlt die Betriebsmeldungsverwaltung. Folgende Meldung wurde nicht verschickt", data);
        }
    }

    //~ SET METHODEN ==========================================================

    /**
     * Die ApplikationsKennung kann mit dieser Methode nachtrglich gesetzt werden.
     *
     * @param applicationLabel ApplikationsKennung - eindeutige Kennung dieser Applikation
     */
    public void setApplicationLabel(String applicationLabel) {
        _applicationLabel = applicationLabel;
    }

    //~ INNERE KLASSEN ========================================================

    /**
     * Diese Klasse fragt den aktuellen Zustand der Sendesteuerung ab. Durch Abfrage der Methode {@link #getState()} kann
     * ermittelt werden, ob eine Betriebsmeldung an die Betriebsmeldungsverwaltung geschickt werden kann, oder nicht.
     */
    private final class OperatingMessageSender implements ClientSenderInterface {
        private byte _state = 1;

        //~ METHODEN ==========================================================

        public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {
            _debug.info("nderung der Sendesteuerung", state);
            _state = state;
        }

        //~ GET METHODEN ======================================================

        public byte getState() {
            return _state;
        }

        public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
            return true;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 30.07.15
