/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.DeviceDefault
 *
 * de.bsvrz.kex.tls.osi7.conversion.DeviceDefault is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.DeviceDefault is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.DeviceDefault.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.DeviceDefault.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.DeviceDefault ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.DeviceDefault wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7.conversion;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.kex.tls.osi7.Device;
import de.bsvrz.kex.tls.osi7.Osi7Telegram;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Standard-Umsetzungsmodul fr Gerte. Wird fr alle Gerte (auch Sm) verwendet, bei denen in der Konfiguration kein
 * spezielles Umsetzungsmodul angegeben wurde.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */

public class DeviceDefault implements DeviceConversionInterface {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Sonstige ---------------------------------------------------------------
	/** DebugLogger fr Debug-Ausgaben */
	private static final Debug debug = Debug.getLogger();
// Private ----------------------------------------------------------------
	/** DaV Interface */
	private final ClientDavInterface _daf;
	/** Objektreferenz auf das bergeordnete Sm-Objekt. */
	private final Device _device;

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/** Erzeugt ein Objekt vom Typ DeviceDefault */
	public DeviceDefault(ClientDavInterface daf, Device device) {
		_daf = daf;
		_device = device;
		debug.finer("DeviceDefault Klasse wurde erfolgreich geladen und instanziert.");
	}

// ------------------------------------------------------------------------
// GETTER / SETTER METHODEN
// ------------------------------------------------------------------------

	/**
	 * Ermglicht den Zugriff auf die aktuelle Datenverteilerverbindung.
	 *
	 * @return Gibt das aktuelle aktuelle Datenverteilerverbindung zurck.
	 */
	public ClientDavInterface getClientDavInterface() {
		return _daf;
	}

// ------------------------------------------------------------------------
// INTERFACE METHODEN
// ------------------------------------------------------------------------


// --------------------- Interface DeviceConversionInterface ---------------------

	/**
	 * Verarbeitet ein Osi7Telegramm. Derzeit keinerlei spezielle Verarbeitung vorgesehen.
	 *
	 * @param osi7Telegram Das zu verarbeitende {@link Osi7Telegram}.
	 */

	public void processOsi7Telegram(Osi7Telegram osi7Telegram) {
		debug.finer("Keine spezielle Verarbeitung fr Osi7Telegramm: " + Debug.NEWLINE + osi7Telegram);
	}
}
