/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.mv.EakFg254FunkKriWeissElectronic
 *
 * de.bsvrz.kex.tls.osi7.conversion.mv.EakFg254FunkKriWeissElectronic is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.mv.EakFg254FunkKriWeissElectronic is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.mv.EakFg254FunkKriWeissElectronic.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.mv.EakFg254FunkKriWeissElectronic.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.mv.EakFg254FunkKriWeissElectronic ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.mv.EakFg254FunkKriWeissElectronic wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7.conversion.mv;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.kex.tls.osi7.*;
import de.bsvrz.kex.tls.osi7.conversion.EakDefault;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Modifiziert fr das Projekt Datenunterzentrale MV/ST durch Logos GmbH
 * (Kristian Strey)
 * - Name des Umsetzungsmoduls = EakFg254FunkKriWeissElectronik
 * - processSingleTelegram, Offsets korrigiert (der lesende Zugriff erfolgt auf
 *   die Datenbytes eines vollstndigen(!) DE-Blocks, das Schreiben hingegen
 *   in den Nutzdatenanteil)
 */

/**
 * Standard-Umsetzungsmodul fr Einzeltelegramme am Eak
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class EakFg254FunkKriWeissElectronic extends EakDefault {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Sonstige ---------------------------------------------------------------
	/**
	 * DebugLogger fr Debug-Ausgaben
	 */
	private static final Debug debug = Debug.getLogger();

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Erzeugt eine neue Instanz von EakFg254KriMvWeissElectronic
	 */
	public EakFg254FunkKriWeissElectronic(ClientDavInterface daf, Sm sm, Eak eak) {
		super(daf, sm, eak);
		debug.finer("EakFg254FunkKriWeissElectronik Klasse wurde erfolgreich geladen und instanziert.");
	}

// ------------------------------------------------------------------------
// INTERFACE METHODEN
// ------------------------------------------------------------------------


// --------------------- Interface EakConversionInterface ---------------------

	/**
	 * Verarbeitet ein EinzelTelegramm. Enthlt das Einzeltelegramm als ersten De-Block einen De-Block fr die
	 * DeKanalNummer 255, so werden die Informationen dieses Blocks hier durch das Eak ausgewertet
	 * (Zeitinformationen und Intervalldaten etc.). Die fr das Einzeltelegramm ermittelten allgemeinen Information
	 * stehen dann in den Umsetzungsmodulen fr die De allen De's des Einzeltelegramms zur Verfgung.
	 * <p/>
	 * Spezielle Anpassung fr die KRI von Weiss-Electronic fr Verwaltung von Telefonlisten: De-Blcke werden
	 * analysiert, ob sie FG 254/ ID 3 / Typ TYP 152 bzw. 153 enthalten. In diesem Fall wird die DE-Nummer (die
	 * immer 0 ist, da DE = KRI gemeint ist) durch die Nummer des Listenplatzes in diesen De-Blcken ersetzt, so
	 * dass in der weiteren Verarbeitung der De-Block dem richtigen DE zugeordnet wird.
	 * <p/>
	 * Spezielle Anpassung fr die KRI von Weiss-Electronic zur Auswertung von Secondary-Stati, die von der KRI fr
	 * die SteuerModule generiert werden. Telegramme werden mit Knotennummer 0, ABsender gestrtes SM (Adressiert
	 * durch korrektes OSI-3-Routingfeld), FG 254 / ID 1 / TYP 1 De-Fehler mit StrSM und StrEAK Bit GLEICHZEITIG
	 * gesetzt und DE-Nummer gleich OSI-2-Adresse des SM bertragen. Damit aufgrund dieser Meldung alle ber diesen
	 * SM betroffenen DE mit ihren Daten auf "Keine Daten" gesetzt werden knnen, werden die StrBits auf StrSM
	 * korrigiert und insbesonder die DE-Nummer auf 0 ( = SM entsprechend TLS) gesetzt.
	 *
	 * @param singleTelegram Das zu verarbeitende Einzeltelegramm {@link SingleTelegram}.
	 */
	public void processSingleTelegram(SingleTelegram singleTelegram) {
		super.processSingleTelegram(singleTelegram);

		// [KS] der lesende Zugriff erfolgt auf die Datenbytes eines vollstndigen(!)
		//      DE-Blocks (getTlsDeDataBlock()), das Schreiben in den Nutzdatenanteil
		//      (getDeData()) => Offset ist unterschiedlich

		// Spezielle Anpassung fr die KRI von Weiss-Electronic fr Verwaltung von Telefonlisten
		for (int i = 0; i < singleTelegram.getCountDeBlock(); i++) {
			DeBlock deBlock = singleTelegram.getDeBlock()[i];
			if (
					(deBlock.getDeChannel() == 0) &&
					(deBlock.getFg() == 254) &&
					(deBlock.getTyp() == 153) &&
					(deBlock.getId() == 3)) {
				// De-Nummer durch Listenplatz ersetzen...
				if (deBlock.getTlsDeDataBlock()[5] == 2) { //Telefonnummernliste
					deBlock.setDeChannel(deBlock.getTlsDeDataBlock()[6]);
				}
				if (deBlock.getTlsDeDataBlock()[5] == 5) { //Secondary-Telefonnummernzuordnung
					deBlock.setDeChannel(deBlock.getTlsDeDataBlock()[7]);
				}
			}
			debug.finest("Verarbeitung durch den EAK: " + Debug.NEWLINE + deBlock.toString());
		}

		// Spezielle Anpassung fr die KRI von Weiss-Electronic zur Auswertung von Secondary-Stati
		for (int i = 0; i < singleTelegram.getCountDeBlock(); i++) {
			DeBlock deBlock = singleTelegram.getDeBlock()[i];
			if (
					(deBlock.getFg() == 254) &&
					(deBlock.getTyp() == 1) &&
					(deBlock.getId() == 1) &&
					(((deBlock.getTlsDeDataBlock()[3]) & 0x3) == 3)) { // Fehler mit StrSM UND StrEAK !! also von KRI
				// Fehlercode auf StrSM setzen
				int _Fehlercode_ = ((deBlock.getTlsDeDataBlock()[3]) & 0xFC) + 2;
				deBlock.getDeData().writeIntAsByte(_Fehlercode_, 0);
				// De-Nummer auf 0 setzen
				deBlock.setDeChannel(0);
			}
			debug.finest("Verarbeitung durch den EAK: " + Debug.NEWLINE + deBlock.toString());
		}
	}
}
